/***********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/

A Java 6 Swing application for color testing and conversions from RGB to
hexadecimal data formats and vice versa.

Copyright (C) 2008-2009, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/
package com.woven_media.colorsafe;

// woven-media.com
import com.woven_media.colorsafe.WindowUtil;
import com.woven_media.colorsafe.IPalette;
import com.woven_media.colorsafe.ColorUtil;
import com.woven_media.colorsafe.NamedColors;
import com.woven_media.colorsafe.X11Colors;
// awt
import java.net.URL;
import java.awt.Font;
import java.awt.Color;
import java.awt.Image;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.BorderLayout;
import java.awt.Container;
// awt.event
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
// util
import java.util.ArrayList;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
// text
import java.text.NumberFormat;
// swingx
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.ImageIcon;

/**
 * ColorModelView class extends <code>javax.swing.JFrame</code> and handles
 * creation of the various color palettes and palette views for the
 * <code>ColorSafe</code> GUI.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 */
public class ColorModelView extends JFrame implements MouseListener, IPalette
{
    private static int ctr, palette, numColors;
    private static JFrame frame;
    private String[] hexColors, namedColors;
    private int[][] rgbColors;
    private JPanel[] panels;
    private int r, g, b;
    private Container cp;
    private String[] labels;
    private GridBagLayout gb;
    private GridBagConstraints gbc;
    private JPanel upperPanel, lowerPanel, swatch;
    private JLabel lblRGB, lblHex;
    private Font labelFont;
    private PaletteInfoPanel pip;
    private Color c;
    private String title;
    private Controller controller;
    private URL url;
    private static ResourceBundle rb;
    private static String errmsg;

    /**
     * ColorModelView constructor handles initialization and sets the palette type.
     *
     * @param c            reference to the <code>Controller</code> object.
     * @param f            reference to the parent frame.
     * @param paletteTitle title of the selected palette.
     * @param paletteType  palette type of the current selection.
     * @since              1.0
     */
    public ColorModelView(Controller c, JFrame f, String paletteTitle, int paletteType)
    {
        super(paletteTitle);
        frame = f;
        controller = c;
        init();
        setPalette(paletteType);
        colorUI();
    }

    /**
     * Handles initialization and the <code>PaletteInfoPanel</code> instantiation.
     *
     * @since 1.0
     */
    private void init()
    {
        Controller.loadIcon(this);
        int num = getNumColors();
        namedColors = new String[num];
        hexColors = new String[num];
        rgbColors = new int[num][3];
        cp = getContentPane();
        cp.setLayout(new BorderLayout());
        pip = new PaletteInfoPanel();
        cp.add(pip, BorderLayout.SOUTH);
        rb = Controller.getResources();
        errmsg = rb.getString("error.outofbounds");
    }

    /**
     * Builds the panel matrix UI.
     *
     * @since 1.0
     */
    public void colorUI()
    {
        getColorMatrix();
        Dimension panelSize = new Dimension(15, 15);
        for(int i = 0; i < getNumColors(); ++i)
        {
            r = rgbColors[i][0];
            g = rgbColors[i][1];
            b = rgbColors[i][2];
            c = new Color(r, g, b);

            panels[i] = new JPanel();
            panels[i].setPreferredSize(panelSize);
            panels[i].setBackground(c);

            panels[i].putClientProperty("RGBArray", new int[] {r, g, b});          // int[]
            panels[i].putClientProperty("RGBColor", c);                            // Color
            panels[i].putClientProperty("HexString", new String(hexColors[i]));    // String
            panels[i].putClientProperty("NameString", new String(namedColors[i])); // String
            panels[i].addMouseListener(this);

            upperPanel.add(panels[i]);
        }
        cp.add(upperPanel, BorderLayout.CENTER);
    }

    /**
     * Determines which palette to build based on constants defined in <code>IPalette</code>
     * interface and the palette type enum passed to the constructor for this instance.
     *
     * @since 1.0
     */
    public void getColorMatrix()
    {
        switch(getPalette())
        {
            case NETSCAPE_PALETTE:
                setNumColors(NETSCAPE_NUM_COLORS);
                safeColorMatrix();
                break;
            case WINDOWS_PALETTE:
                setNumColors(WIN_NUM_COLORS);
                safeColorMatrix();
                break;
            case WINDOWS_GRAYS_PALETTE:
                setNumColors(WIN_GRAYS_NUM_COLORS);
                grayColorMatrix();
                break;
            case GRAYSCALE_PALETTE:
                setNumColors(GRAYSCALE_NUM_COLORS);
                grayColorMatrix();
                break;
            case NAMED_COLORS_PALETTE:
                setNumColors(NAMED_NUM_COLORS);
                namedColorMatrix(NAMED_COLORS_PALETTE);
                break;
            case X11_PALETTE:
                setNumColors(X11_NUM_COLORS);
                namedColorMatrix(X11_PALETTE);
                break;
        }
    }

    /**
     * Builds either the <code>Windows System Palette</code> or <code>Netscape Palette</code> based
     * on the value of <code>this.getPalette</code> method.
     *
     * @since 1.0
     */
    public void safeColorMatrix()
    {
        init();
        ctr = 0;
        int sel = getPalette();
        int num = getNumColors();
        int[] rgbSafeDigits = new int[] { 0, 51, 102, 153, 204, 255 };
        String[] hexSafeDigits = new String[] { "00", "33", "66", "99", "CC", "FF" };
        panels = new JPanel[num];
        upperPanel = new JPanel(new GridLayout(12, 18, 1, 1));

        if(sel == NETSCAPE_PALETTE || sel == WINDOWS_PALETTE)
        {
            for(int i=0; i < 6; ++i)
            {
                for(int j=0; j < 6; ++j)
                {
                    for(int k=0; k < 6; ++k)
                    {
                        try
                        {
                            rgbColors[ctr][0] = (sel == WINDOWS_PALETTE) ? rgbSafeDigits[k] : rgbSafeDigits[i];
                            rgbColors[ctr][1] = rgbSafeDigits[j];
                            rgbColors[ctr][2] = (sel == WINDOWS_PALETTE) ? rgbSafeDigits[i] : rgbSafeDigits[k];
                            hexColors[ctr] = hexSafeDigits[i] + hexSafeDigits[j] + hexSafeDigits[k];
                            namedColors[ctr] = "NULL";
                            ctr++;
                        }
                        catch(ArrayIndexOutOfBoundsException ae)
                        {
                            System.err.println(errmsg + "in ColorModelView#safeColorMatrix"); // I18N
                        }
                    } // ...
                } // ...
            } // end 1st nest
        } // end if
    }

    /**
     * Builds either the <code>Windows 256 Grays Palette</code> or standard <code>Grayscale Palette</code>
     * based on the value of the <code>this.getPalette</code> method.
     *
     * @since 1.0
     */
    public void grayColorMatrix()
    {
        init();
        int sel = getPalette();
        int num = getNumColors();
        panels = new JPanel[num];
        upperPanel = new JPanel(new GridLayout(15, 18, 1, 1));

        if(sel == WINDOWS_GRAYS_PALETTE || sel == GRAYSCALE_PALETTE)
        {
            for(int i = 0; i < num; ++i)
            {
                try
                {
                    rgbColors[i][0] = i;
                    rgbColors[i][1] = i;
                    rgbColors[i][2] = i;
                    hexColors[i] = ColorUtil.RGBtoHEX(i, i, i);
                    namedColors[i] = "NULL";
                }
                catch(ArrayIndexOutOfBoundsException ae)
                {
                    System.err.println(errmsg + "in ColorModelView#grayColorMatrix"); // I18N
                }
            }
        }
    }

    /**
     * Builds the <code>NamedColors</code> palette. Calls <code>NamedColors</code> class.
     *
     * @param type indicates Windows-named color palette or standards-compliant
     *             <code>X11</code> color palette.
     * @since      1.0
     */
    public void namedColorMatrix(int type)
    {
        init();
        int num = 0;
        ArrayList<String[]> names = null;

        switch(type)
        {
            case NAMED_COLORS_PALETTE:
                names = NamedColors.createList();
                break;
            case X11_PALETTE:
                names = X11Colors.createList();
                break;
        }

        num = names.size();
        upperPanel = new JPanel(new GridLayout(8, 18, 1, 1));
        panels = new JPanel[num];

        for(int i = 0; i < num; ++i)
        {
            try
            {
                String[] values = (String[]) names.get(i);
                int[] rgb = ColorUtil.buildRGBarray(values[1]);

                rgbColors[i][0] = rgb[0];
                rgbColors[i][1] = rgb[1];
                rgbColors[i][2] = rgb[2];
                namedColors[i] = values[0];
                hexColors[i] = values[1];
            }
            catch(ArrayIndexOutOfBoundsException ae)
            {
                System.err.println(errmsg + "in ColorModelView#namedColorMatrix"); // I18N
            }
        }
    }

    /**
     * Sets the number of colors for this palette.
     *
     * @param num <code>int</code> value representing the number of colors the current palette.
     * @since     1.0
     */
    public void setNumColors(int num)
    {
        numColors = num;
    }

    /**
     * Returns the number of colors for this palette.
     *
     * @return numColors <code>int</code> value representing to number of colors in current palette.
     * @since  1.0
     */
    public int getNumColors()
    {
        return numColors;
    }

    /**
     * Sets the palette type member.
     *
     * @param palette <code>int</code> value representing the palette type.
     * @since         1.0
     */
    public void setPalette(int palette)
    {
        this.palette = palette;
    }

    /**
     * Returns the palette type.
     *
     * @return palette an <code>int</code> value representing the palette type.
     * @since          1.0
     */
    public int getPalette()
    {
        return palette;
    }

    /**
     * Handles mouse events entering individual color panels. Grabs the client
     * properties for this color panel and sets the <code>PaletteInfoPanel#selectedObject</code>
     * method.
     *
     * @param e <code>java.awt.event.MouseEvent</code> parameter.
     * @since   1.0
     */
    public void mouseEntered(MouseEvent e)
    {
        setCursor(new Cursor(Cursor.HAND_CURSOR));
        if(e.getSource() instanceof JPanel)
        {
            JPanel selectedPanel = (JPanel) e.getComponent();
            Color c = (Color) selectedPanel.getClientProperty("RGBColor");
            String hex = (String) selectedPanel.getClientProperty("HexString");
            String name =(String) selectedPanel.getClientProperty("NameString");
            int[] rgb = (int[]) selectedPanel.getClientProperty("RGBArray");
            pip.selectedObject(c, hex, rgb, name);
            pip.repaint();
        }
    }

    /**
     * Sets the <code>java.awt.Color</code> object by grabbing the client property
     * <code>"RGBColor"</code> from the <code>javax.swing.JComponent.getClientProperty</code>
     * method for the selected color panel. Sets the current color and disposes of the palette
     * frame.
     *
     * @param e <code>java.awt.event.MouseEvent</code>.
     * @see     javax.swing.JComponent#getClientProperty
     * @since   1.0
     */
    public void mouseReleased(MouseEvent e)
    {
        if(e.getSource() instanceof JPanel)
        {
            JPanel selectedPanel = (JPanel) e.getComponent();
            Color c = (Color) selectedPanel.getClientProperty("RGBColor");
            controller.setCurrentColor(c);
            this.setVisible(false);
            this.dispose();
        }
    }

    /**
     * Reverts back to default cursor when mouse exits a color panel.
     *
     * @param e <code>java.awt.event.MouseEvent</code> required parameter.
     */
    public void mouseExited(MouseEvent e)
    {
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    /**
     * Empty method signature required by the <code>java.awt.event.MouseListener</code> interface.
     *
     * @param e <code>java.awt.event.MouseEvent</code> required parameter.
     */
    public void mouseClicked(MouseEvent e) {}

    /**
     * Empty method signature required by the <code>java.awt.event.MouseListener</code> interface.
     *
     * @param e <code>java.awt.event.MouseEvent</code> required parameter.
     */
    public void mousePressed(MouseEvent e) {}

} // End class

