/***********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/

A Java 6 Swing application for color design, testing and conversions
from RGB/HSB/CMYK to hexadecimal data formats and vice versa.

Copyright (C) 2008-2009, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/

//
// TODO: Start extending the listener classes and
// seperate out the behaviors from the UI components.
// This will offer much more flexibility in implementing
// complex behaviors in the menu systems. An Observer
// pattern would make sense here as well. A great deal of
// change is occuring within the seperate menu functions
// and encapsulating these behaviors and loose-coupling
// of objects makes sense.
//
// see comments marked --------- Behavior ---------
//
// -Brent Parrish 12 May 2008
//
package com.woven_media.colorsafe;

import com.woven_media.colorsafe.WindowUtil;
import com.woven_media.colorsafe.IPalette;

import java.io.IOException;
import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.SwingUtilities;
import javax.swing.JRootPane;
import javax.swing.UIManager;
import javax.swing.LookAndFeel;
import javax.swing.event.MenuListener;
import javax.swing.event.MenuEvent;
import javax.swing.UIManager.LookAndFeelInfo;


/**
 * ColorSafeMenuBar class creates a top-level menubar component and extends
 * <code>javax.swing.JMenuBar</code>.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 * @see     javax.swing.JMenuBar
 */
public class ColorSafeMenuBar extends JMenuBar implements IPalette
{
    private static ResourceBundle rb;
    private Controller controller;
    private JFrame frame;

    /**
     * Class constructor handles initialization and starts the menu building process
     * by calling <code>ColorSafeMenuBar#buildMenus</code>.
     *
     * @param c reference to the <code>Controller</code> class.
     * @param f reference to the parent window.
     * @since   1.0
     */
    public ColorSafeMenuBar(Controller c, JFrame f)
    {
        controller = c;
        frame = f;
        rb = Controller.getResources();
        buildMenus();
    }

    /**
     * Builds all the top-level menus and sets the <code>javax.swing.JMenuBar</code>.
     *
     * @since 1.0
     */
    protected void buildMenus()
    {
        setOpaque(true);
        setBorderPainted(true);

        JMenu file = buildFileMenu();
        JMenu edit = buildEditMenu();
        JMenu palette = buildViewMenu();
        JMenu help = buildHelpMenu();

        add(file);
        add(edit);
        edit.setMnemonic('E');
        add(palette);
        add(help);

        frame.setJMenuBar(this);
    }

    /**
     * Builds the <code>File</code> menu.
     *
     * @return file <code>javax.swing.JMenu</code> of the top-level <code>File</code> menu.
     * @since       1.0
     */
    protected JMenu buildFileMenu()
    {
        JMenu file = new JMenu(rb.getString("menu.file"));         // I18N
        JMenuItem quit = new JMenuItem(rb.getString("file.quit")); // I18N
        quit.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                System.exit(0);
            }
        });
        //quit.setFont(new Font("Arial", Font.PLAIN, 12));
        file.add(quit);

        // -----------------

        file.addSeparator();
        JMenu choose = new JMenu(rb.getString("file.choose")); // I18N

        // --------- Behavior ---------
        // ...hmmmm...I'm smellin factory here! i.e. LanguageFactory.setMenuItem(...
        // Language JRadioButtonMenuItem Group
        ButtonGroup languages = new ButtonGroup();

        // English option
        // NOTE: The default resources path should be set in one place ONLY!
        //       We also need resource file management at this point.
        URL url = java.net.URLClassLoader.getSystemResource("resources/default/flags/us.gif");
        Icon flagIcon = new ImageIcon(url); // <-- Icon Adapter??? see Dr. Jamf
        JRadioButtonMenuItem en = new JRadioButtonMenuItem(rb.getString("file.en"), flagIcon); // I18N
        en.setActionCommand("US");  // Set the ActionCommand to the two-letter country code UPPERCASE!
        setLanguageOption(en, "en");// Set the language option to the two-letter language code
        languages.add(en);
        choose.add(en);

        // German option
        url = java.net.URLClassLoader.getSystemResource("resources/default/flags/de.gif");
        flagIcon = new ImageIcon(url);
        JRadioButtonMenuItem de = new JRadioButtonMenuItem(rb.getString("file.de"), flagIcon); // I18N
        de.setActionCommand("DE");
        setLanguageOption(de, "de");
        languages.add(de);
        choose.add(de);
        // -----------------
        file.add(choose);
        return file;
    }

    /**
     * Builds the <code>Edit</code> menu and attachs event listeners to the menu items.
     * The <code>Edit</code> menu opens the nearest safe color or the current selected
     * color within the <code>javax.swing.JColorChooser</code> dialog.
     *
     * @return edit <code>javax.swing.JMenu</code> of the top-level <code>Edit</code> menu.
     * @since 1.0
     */
    protected JMenu buildEditMenu()
    {

        final JMenuItem nearest = new JMenuItem(rb.getString("edit.nearest") + "..."); // I18N
        JMenuItem current = new JMenuItem(rb.getString("edit.current") + "...");       // I18N
        JMenu edit = new JMenu(rb.getString("menu.edit"));                             // I18N
        JMenuItem bg = new JMenuItem(rb.getString("edit.background") + "...");         // I18N

        // --------- Behavior ---------
        edit.addMenuListener(new MenuListener()
        {
            public void menuSelected(MenuEvent e)     // required by interface
            {
                nearest.setEnabled(!controller.isSafeColor());
            }
            public void menuDeselected(MenuEvent e){} // required by interface
            public void menuCanceled(MenuEvent e){}   // required by interface
        });

        KeyStroke ctrlP = KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_DOWN_MASK); // Ctrl + P
        KeyStroke ctrlN = KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_DOWN_MASK); // Ctrl + N
        // Current selected color
        current.setAccelerator(ctrlP); // Assign Ctrl + P
        // --------- Behavior ---------
        current.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {   // Opens color chooser with current selected color.
                // >>> TODO: 2nd param should be a constant
                controller.colorPicker(rb.getString("picker.current"), RGB_VAL); // I18N
            }
        });
        // Nearest safe color
        nearest.setAccelerator(ctrlN); // Assign Ctrl + N
        // --------- Behavior ---------
        nearest.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {   // Opens color chooser and selects nearest safe color.
                controller.colorPicker(rb.getString("picker.nearest"), HEX_VAL); // I18N
            }
        });
         // Opens the background color picker
        bg.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                controller.colorPicker("Change background color", BG_VAL) ; // I18N
            }
        });

        // Add menus to collection
        edit.add(current);
        edit.add(nearest);
        edit.addSeparator();
        edit.add(bg);

        return edit;
    }

    /**
     * Builds the <code>View</code> menu and attachs event listeners to the menu items.
     * The <code>View</code> menu provides access to the palette windows.
     *
     * @return color <code>javax.swing.JMenu</code> of the top-level <code>View</code> menu.
     * @since        1.0
     */
    protected JMenu buildViewMenu()
    {
        KeyStroke ctrlB = KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK); // Ctrl + B
        KeyStroke ctrlG = KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_DOWN_MASK); // Ctrl + G
        KeyStroke ctrlX = KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.ALT_DOWN_MASK);  // Alt + X
        // menus
        JMenu color = new JMenu(rb.getString("menu.palette"));                         // I18N
        JMenu palettes = new JMenu(rb.getString("palette.submenu"));                   // I18N
        // items
        JMenuItem netscape = new JMenuItem(rb.getString("palette.netscape") + "...");  // I18N
        JMenuItem windows = new JMenuItem(rb.getString("palette.windows") + "...");    // I18N
        JMenuItem grays = new JMenuItem(rb.getString("palette.grays") + "...");        // I18N
        JMenuItem named = new JMenuItem(rb.getString("palette.named") + "...");        // I18N
        JMenuItem x11 = new JMenuItem(rb.getString("palette.x11") + "...");            // I18N

         // PluggableLookAndFeel (PLAF) Items
        JMenuItem win = new JMenuItem("Windows");            // DO NOT CHANGE NAMES!!!
        JMenuItem motif = new JMenuItem("CDE/Motif");            // ""
        JMenuItem metal = new JMenuItem("Metal");                // ""
        JMenuItem classic = new JMenuItem("Windows Classic");    // ""
        // Set LAF properties
        // TODO: use the WinUtil.getClassName method instead of the qualified package string.
        win.putClientProperty("class", "com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
        classic.putClientProperty("class", "com.sun.java.swing.plaf.windows.WindowsClassicLookAndFeel");
        metal.putClientProperty("class", "javax.swing.plaf.metal.MetalLookAndFeel");
        motif.putClientProperty("class", "com.sun.java.swing.plaf.motif.MotifLookAndFeel");
        // Put menu items in a collection
        final JMenuItem[] lafItems = {win, motif, metal, classic};
        final UIManager.LookAndFeelInfo[] lafs = UIManager.getInstalledLookAndFeels();
        // LAF(Skins) submenu
        final JMenu plaf = new JMenu("Skins"); // I18N
        plaf.addMenuListener(new MenuListener() // --------- Behavior ---------
        {
            public void menuSelected(MenuEvent e)      // required by interface
            {
                final String[] result = UIManager.getLookAndFeel().getClass().toString().split("[ ]");
                final String currentLAF = result[1];
                for(int i = 0; i < lafs.length; ++i)
                {
                    String lnfInfo = lafs[i].getClassName().toString();
                    if(lnfInfo.equals(currentLAF))
                    {
                        for(int j = 0; j < lafItems.length; ++j)
                        {
                            if(lafItems[j].getClientProperty("class").equals(currentLAF))
                            {
                                lafItems[j].setEnabled(false);
                            }
                            else
                            {
                                lafItems[j].setEnabled(true);
                            }
                        } // End inner for
                    } // End if
                } // End outer for
            } // End required
            public void menuDeselected(MenuEvent e){} // required by interface
            public void menuCanceled(MenuEvent e){}   // required by interface
        });
        // Opens color safe palette
        netscape.setAccelerator(ctrlB);
        // --------- Behavior ---------
        netscape.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                controller.openColorPalette(rb.getString("palette.netscape"), NETSCAPE_PALETTE); // I18N
            }
        });
        // Opens Windows System palette
        windows.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                controller.openColorPalette(rb.getString("palette.windows") , WINDOWS_PALETTE); // I18N
            }
        });
        // Opens grayscale palette
        grays.setAccelerator(ctrlG);
        grays.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                controller.openColorPalette(rb.getString("palette.grays"), GRAYSCALE_PALETTE); // I18N
            }
        });
        // Opens Windows Named colors palette
        named.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                controller.openColorPalette(rb.getString("palette.named"), NAMED_COLORS_PALETTE); // I18N
            }
        });
        // Opens X11 colors palette
        x11.setAccelerator(ctrlX);
        x11.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                controller.openColorPalette(rb.getString("palette.x11"), X11_PALETTE); // I18N
            }
        });
        // Metal LAF
        metal.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                WindowUtil.setJavaLookAndFeel();
                updateComponentUI();
            }
        });
        // Motif LAF
        motif.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                WindowUtil.setMotifLookAndFeel();
                updateComponentUI();
            }
        });
        // Windows LAF
        win.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                WindowUtil.setNativeLookAndFeel();
                updateComponentUI();
            }
        });
        // Windows Classic LAF
        classic.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                WindowUtil.setClassicLookAndFeel();
                updateComponentUI();
            }
        });
        // Add palette items to the sub-menu
        palettes.add(netscape);
        palettes.add(grays);
        palettes.add(named);
        palettes.add(windows);
        palettes.add(x11);
        // LAF submenu
        plaf.add(motif);
        plaf.add(metal);
        plaf.add(win);
        plaf.add(classic);
        // Add submenu to main
        color.add(palettes);
        color.addSeparator();
        color.add(plaf);

        return color;
    }

    /**
     * Builds the <code>Help</code> menu and provides access to the <code>About</code> box.
     *
     * @return file <code>javax.swing.JMenu</code> of the top-level <code>Help</code> menu.
     * @since 1.0
     */
    protected JMenu buildHelpMenu()
    {
        KeyStroke ctrlH = KeyStroke.getKeyStroke(KeyEvent.VK_H, InputEvent.CTRL_DOWN_MASK); // Ctrl + H

        final JMenu help = new JMenu(rb.getString("menu.help"));                     // I18N
        final JMenuItem about = new JMenuItem(rb.getString("help.about") + "...");   // I18N
        JMenuItem topics = new JMenuItem(rb.getString("help.topics") + "...");       // I18N
        topics.setAccelerator(ctrlH);                                                // Assign Ctrl + H
        // --------- Behavior ---------
        // NOTE:
        // Behaviors encapsulated here are ok. This will
        // not change too much in the future.
        topics.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                // Will implement in ColorSafe version 1.0 rc1
            }
        });
        // Open the About box.
        about.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    controller.showAboutBox();
                }
                catch(IOException ioe)
                {
                    ioe.printStackTrace();
                }
            }
        });
        // Will add in Colorsafe version 1.0 rc1
        // help.add(topics);
        //help.addSeparator();
        help.add(about);

        return help;
    }

    /**
     * Updates the component UI tree when this LookAndFeel changes.
     *
     * @see       javax.swing.SwingUtilities#updateComponentTreeUI
     * @since 1.0
     */
    private void updateComponentUI()
    {
        ColorSafeUI.updateComponentUI(frame);
        frame.pack();
    }

    /**
     * Sets the language option at runtime.
     *
     * @param item a <code>javax.swing.JRadioButtonMenuItem</code> for language selection.
     * @param lang standard two-letter string language code.
     * @since      1.0
     */
    public void setLanguageOption(final JRadioButtonMenuItem item, final String lang)
    {
        if(controller.getLocale().getLanguage() == lang)
        {
            item.setSelected(true);
        }
        item.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                controller.setLocale(new Locale(lang, item.getActionCommand()));
                Color saved = controller.getCurrentSelectedColor(); // Save color before restart
                controller.restart();
                controller.setCurrentColor(saved); // load back saved color
            }
        });
    }

} // End class
